//------------------------------------------------------------------------------
//  Author:    	MXF Team
//  Comment:	MXF_TK: This class defines the interface to set the descriptors.
//
//  The following convention is used for defining parameters and retrun values:
//      in : input parameter (will be used for computation in the function)
//      out : output parameter (will be modified by the function call)
//      in-out : input parameter that may be modified by the function call.
//      free : MXFTk user is reponsible for the deletion of this parameter
//             unless it is passed as a parameter of another function without 
//             the "free" tag.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------

#ifndef OC_I_MXF_FILE_DESCRIPTOR_HPP
#define OC_I_MXF_FILE_DESCRIPTOR_HPP

#include <mxf_tk/mxf_tk.hpp>

namespace opencube
{
  namespace MXFTK_NAMESPACE
  {
    enum subdescriptor_t
    {
      undefined_descriptor=0,
      jpeg2000_picture
    };


    class I_mxf_subdescriptor
    {
    public:
      virtual subdescriptor_t CALLING_CONVENTION get_subdescriptor_type() const = 0;
    };


    class I_mxf_jpeg2000_picture_subdescriptor : public I_mxf_subdescriptor
    {
    public:
      virtual void CALLING_CONVENTION set_r_siz(const uint16_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_x_siz(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_y_siz(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_xo_siz(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_yo_siz(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_xt_siz(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_yt_siz(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_xto_siz(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_yto_siz(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_c_siz(const uint16_t /* in */) = 0;
      virtual void CALLING_CONVENTION add_picture_component_sizing(const uint8_t /* in */, 
                                                                   const uint8_t  /* in */, 
                                                                   const uint8_t  /* in */) = 0;
      virtual void CALLING_CONVENTION set_coding_style_default(const uint8_t * /* in free */, 
                                                               const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_quantization_default(const uint8_t * /* in */, 
                                                               const uint32_t /* in free */) = 0;
    };


    class I_mxf_file_descriptor
    {
    public:
      virtual void CALLING_CONVENTION set_sample_rate(const uint32_t numerator /* in */, 
                                                      const uint32_t denominator /* in */) = 0;
      virtual void CALLING_CONVENTION set_container_duration(const uint64_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_essence_container_ul(const char * /* in free */) = 0;
      virtual void CALLING_CONVENTION set_codec_ul(const char * /* in free */) = 0;

      // After a subdescriptor is added to descriptor, subdescriptor id deleted by descriptor
      // user won't have to delete it
      virtual void CALLING_CONVENTION add_subdescriptor(const I_mxf_subdescriptor * /* in */) = 0;
    };


    class I_mxf_generic_picture_essence_descriptor : public I_mxf_file_descriptor
    {
    public:
      virtual void CALLING_CONVENTION set_signal_standard(const uint8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_frame_layout(const uint8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_stored_width(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_stored_height(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_stored_f2_offset(const int32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_sampled_width(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_sampled_height(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_sampled_x_offset(const int32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_sampled_y_offset(const int32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_display_width(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_display_height(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_display_x_offset(const int32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_display_y_offset(const int32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_display_f2_offset(const int32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_aspect_ratio(const uint32_t /* in */, 
                                                       const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_active_format_descriptor(const uint8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_video_line_map(const int32_t /* in */, 
                                                         const int32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_alpha_transparency(const uint8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_capture_gamma_ul(const char * /* in free */) = 0;
      virtual void CALLING_CONVENTION set_image_alignment_offset(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_image_start_offset(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_image_end_offset(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_field_dominance(const uint8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_picture_essence_coding_ul(const char * /* in free */) = 0;
    };


    class I_mxf_cdci_picture_essence_descriptor : public I_mxf_generic_picture_essence_descriptor
    {
    public:
      virtual void CALLING_CONVENTION set_component_depth(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_horizontal_subsampling(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_vertical_subsampling(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_color_siting(const uint8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_reversed_byte_order(const bool /* in */) = 0;
      virtual void CALLING_CONVENTION set_padding_bits(const int16_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_alpha_sample_depth(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_black_ref_level(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_white_ref_level(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_color_range(const uint32_t /* in */) = 0;
    };


    class I_mxf_mpeg2_video_descriptor : public I_mxf_cdci_picture_essence_descriptor
    {
    public:
      virtual void CALLING_CONVENTION set_single_sequence(const bool /* in */) = 0;
      virtual void CALLING_CONVENTION set_constant_b_frames(const bool /* in */) = 0;
      virtual void CALLING_CONVENTION set_coded_content_type(const uint8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_low_delay(const bool /* in */) = 0;
      virtual void CALLING_CONVENTION set_closed_gop(const bool /* in */) = 0;
      virtual void CALLING_CONVENTION set_identical_gop(const bool /* in */) = 0;
      virtual void CALLING_CONVENTION set_max_gop(const uint16_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_b_picture_count(const uint16_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_bit_rate(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_profile_and_level(const uint8_t /* in */) = 0;
    };


    class I_mxf_rgba_picture_essence_descriptor : public I_mxf_generic_picture_essence_descriptor
    {
    public:
      virtual void CALLING_CONVENTION set_component_max_ref(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_component_min_ref(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_alpha_max_ref(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_alpha_min_ref(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_scanning_direction(const uint8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_pixel_layout(const uint8_t * /* in free */, 
                                                       const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_palette(const uint8_t * /* in free */, 
                                                  const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_palette_layout(const uint8_t * /* in free */, 
                                                         const uint32_t /* in */) = 0;
    };


    class I_mxf_generic_sound_essence_descriptor : public I_mxf_file_descriptor
    {
    public:
      virtual void CALLING_CONVENTION set_audio_sampling_rate(const uint32_t /* in */, 
                                                              const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_locked_unlocked(const bool /* in */) = 0;
      virtual void CALLING_CONVENTION set_audio_ref_level(const int8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_electro_spatial_formulation(const uint8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_channel_count(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_quantization_bits(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_dial_norm(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_sound_essence_compression_ul(const char * /* in free */) = 0;
    };


    class I_mxf_wave_audio_essence_descriptor : public I_mxf_generic_sound_essence_descriptor
    {
    public:
      virtual void CALLING_CONVENTION set_block_align(const uint16_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_sequence_offset(const uint8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_avg_bps(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_channel_assignment_ul(const char * /* in free */) = 0;
      virtual void CALLING_CONVENTION set_peak_envelope_version(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_peak_envelope_format(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_points_per_peak_value(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_peak_envelope_block_size(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_peak_channels(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_peak_frames(const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_peak_of_peaks_position(const uint64_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_peak_envelope_timestamp(const uint64_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_peak_envelope_data(const uint8_t * /* in free */, 
                                                             const uint32_t /* in */) = 0;
    };


    class I_mxf_aes3_audio_essence_descriptor : public I_mxf_wave_audio_essence_descriptor
    {
    public:
      virtual void CALLING_CONVENTION set_emphasis(const uint8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_block_start_offset(const uint16_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_aux_bits_mode(const uint8_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_channel_status_mode(const uint8_t * /* in free */, 
                                                              const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_fixed_channel_status_data(const uint8_t**  /* in free */, 
                                                                    const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_user_data_mode(const uint8_t * /* in free */, 
                                                         const uint32_t /* in */) = 0;
      virtual void CALLING_CONVENTION set_fixed_user_data(const uint8_t ** /* in free */, 
                                                          const uint32_t /* in */) = 0;
    };


    class I_mxf_generic_data_essence_descriptor : public I_mxf_file_descriptor
    {
    public:
      virtual void CALLING_CONVENTION set_data_essence_coding_ul(const char * /* in free */) = 0;
    };


    class I_mxf_vbi_data_essence_descriptor : public I_mxf_generic_data_essence_descriptor
    {
    };


    class I_mxf_anc_data_essence_descriptor : public I_mxf_generic_data_essence_descriptor
    {
    };


    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMxfFileDescriptorInterface(I_mxf_file_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMxfFileDescriptorInterface(I_mxf_file_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMxfGenericPictureEssenceDescriptorInterface(I_mxf_generic_picture_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMxfGenericPictureEssenceDescriptorInterface(I_mxf_generic_picture_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMxfCDCIPictureEssenceDescriptorInterface(I_mxf_cdci_picture_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMxfCDCIPictureEssenceDescriptorInterface(I_mxf_cdci_picture_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMxfMpeg2VideoDescriptorInterface(I_mxf_mpeg2_video_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMxfMpeg2VideoDescriptorInterface(I_mxf_mpeg2_video_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMxfRGBAPictureEssenceDescriptorInterface(I_mxf_rgba_picture_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMxfRGBAPictureEssenceDescriptorInterface(I_mxf_rgba_picture_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMxfGenericSoundEssenceDescriptorInterface(I_mxf_generic_sound_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMxfGenericSoundEssenceDescriptorInterface(I_mxf_generic_sound_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMxfWaveAudioEssenceDescriptorInterface(I_mxf_wave_audio_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMxfWaveAudioEssenceDescriptorInterface(I_mxf_wave_audio_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMxfAes3AudioEssenceDescriptorInterface(I_mxf_aes3_audio_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMxfAes3AudioEssenceDescriptorInterface(I_mxf_aes3_audio_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMxfGenericDataEssenceDescriptorInterface(I_mxf_generic_data_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMxfGenericDataEssenceDescriptorInterface(I_mxf_generic_data_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMxfVbiDataEssenceDescriptorInterface(I_mxf_vbi_data_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMxfVbiDataEssenceDescriptorInterface(I_mxf_vbi_data_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMxfAncDataEssenceDescriptorInterface(I_mxf_anc_data_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMxfAncDataEssenceDescriptorInterface(I_mxf_anc_data_essence_descriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMxfJpeg2000PictureSubdescriptorInterface(I_mxf_jpeg2000_picture_subdescriptor** /* out free */);

    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMxfJpeg2000PictureSubdescriptorInterface(I_mxf_jpeg2000_picture_subdescriptor** /* out free */);
  }
}

#endif
