//------------------------------------------------------------------------------
//  Author:    	MXF Team
//  Comment:	MXF_TK: This class defines the xdcam proxy file interface.
//				XDCam is a SONY Camcorder allowing recording and writing
//				of MXF files with DV / AES audio material. Use this
//				class to build MXF file having the same properties 
//				than those produced by the XDCam.
//
//  The following convention is used for defining parameters and retrun values:
//      in : input parameter (will be used for computation in the function)
//      out : output parameter (will be modified by the function call)
//      in-out : input parameter that may be modified by the function call.
//      free : MXFTk user is reponsible for the deletion of this parameter
//             unless it is passed as a parameter of another function without 
//             the "free" tag.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#ifndef OC_I_XDCAM_DV_FILE_HPP
#define OC_I_XDCAM_DV_FILE_HPP

#include <mxf_tk/mxf_tk.hpp>
#include <mxf_tk/I_op1a_file.hpp>

namespace opencube
{
  namespace MXFTK_NAMESPACE
  {
    /* I_xdcam_dv_file:
     * An interface to create an xdcam dv file.
     */
    class I_xdcam_dv_file : public I_op1a_file
    {
    private:
      virtual bool CALLING_CONVENTION set_external_material(I_concrete_material*, const wchar_t*)=0;
    };

    /*
     * Retrieves an xdcam_dv_file interface.
     * I_xdcam_dv_file** pointer to the newly allocated object
     * const char* path to the xdcam file to be written.
     * Call to flush() will effectively cause the creation of
     * the file.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewXdcamDvFileInterface(I_xdcam_dv_file** /* out free */, 
                                                                          const char* /* in free */);

    /*
     * Retrieves an xdcam_dv_file interface in streaming mode.
     * I_xdcam_dv_file** pointer to the newly allocated object
     * const char* path to the xdcam file to be written.
     * Call to flush() will effectively cause the creation of
     * the file.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewXdcamDvStreamInterface(I_xdcam_dv_file** /* out free */, 
                                                                            I_output_mxf_stream_task* /* in free */);

    /*
     * Frees an xdcam_dv_file interface.
     * I_xdcam_dv_file** pointer to the object to be freed.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeXdcamDvFileInterface(I_xdcam_dv_file** /* in out */);
  }
}

#endif
