//------------------------------------------------------------------------------
//  Author:    	MXF Team
//  Comment:	MXF_TK: This class defines the OpAtom file interface.
//            It can be used to generate Opatoms with any legal audiovisual
//            material.
//
//  The following convention is used for defining parameters and retrun values:
//      in : input parameter (will be used for computation in the function)
//      out : output parameter (will be modified by the function call)
//      in-out : input parameter that may be modified by the function call.
//      free : MXFTk user is reponsible for the deletion of this parameter
//             unless it is passed as a parameter of another function without 
//             the "free" tag.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#ifndef OC_I_OPATOM_FILE_HPP
#define OC_I_OPATOM_FILE_HPP

#include <mxf_tk/mxf_tk.hpp>
#include <mxf_tk/I_op1a_file.hpp>

namespace opencube
{
  namespace MXFTK_NAMESPACE
  {
    /* P2 format:
       List of all the video formats supported by the P2 camcorder
     */
    enum p2_format
    {
      // based on 480/60i
      p2_480i_60i_dviec,
      p2_480i_60i_dvcpro25,
      p2_480i_60i_dvcpro50,
      p2_480i_30p_2_2_pulldown_dviec,
      p2_480i_30p_2_2_pulldown_dvcpro25,
      p2_480i_30p_2_2_pulldown_dvcpro50,
      p2_480i_24p_2_3_pulldown_dviec,
      p2_480i_24p_2_3_pulldown_dvcpro25,
      p2_480i_24p_2_3_pulldown_dvcpro50,
      p2_480i_24p_2_3_3_2_pulldown_dviec,
      p2_480i_24p_2_3_3_2_pulldown_dvcpro25,
      p2_480i_24p_2_3_3_2_pulldown_dvcpro50,
      // based on 576/50i
      p2_576i_50i_dviec,
      p2_576i_50i_dvcpro25,
      p2_576i_50i_dvcpro50,
      p2_576i_25p_2_2_pulldown_dviec,
      p2_576i_25p_2_2_pulldown_dvcpro25,
      p2_576i_25p_2_2_pulldown_dvcpro50,
      // based on 720/60p
      p2_720p_60p_dvcpro100,
      p2_720p_30p_2_2_pulldown_dvcpro100,
      p2_720p_24p_2_3_pulldown_dvcpro100,
      p2_720p_30p_native_dvcpro100,
      p2_720p_24p_native_dvcpro100,
      // based on 720/50p
      p2_720p_50p_dvcpro100,
      p2_720p_25p_2_2_pulldown_dvcpro100,
      p2_720p_25p_native_dvcpro100,
      // based on 1080/60i
      p2_1080i_60i_dvcpro100,
      p2_1080i_30p_2_2_pulldown_dvcpro100,
      p2_1080i_24p_2_3_pulldown_dvcpro100,
      p2_1080i_24p_2_3_3_2_pulldown_dvcpro100,
      // based on 1080/50i
      p2_1080i_50i_dvcpro100,
      p2_1080i_25p_2_2_pulldown_dvcpro100,
      p2_unknown
    };

    /* NewP2Shot
     * This function lets you fill a P2 "Contents" directory with MXF P2 files
     * similar to the ones the P2 Camcorder would produce.
     * The first parameter is listing the input files to be MXF wrapped (this
     * should be one DV and 2 or 4 AES files). 
     * The second parameter is the complete path toward the output "Contents" directory
     * The third parameter is the basename for this shot (that will be used
     * to set up the names of the MXF files) and should be EXACTLY 6 characters long (+ '\0' not counted)
     * API user can alos the start timecode and umid for this shot or leave this parameter to NULL
     * and the API will generate default ones.
     * The fourth parameter specifies the P2 format
     * The fifth parameter defines the start timecode of the sequence
     * The sixth parameter defines the umid of the generic material
     * Returns !=0 if the function operated sucessfully. The returned uint32_tshould then be used as parameter 
     * of WaitEndNewP2Shot, CancelNewP2Shot and ProgressNewP2Shot
     * WARNING! You should call WaitEndNewP2Shot after calling this function to make sure the process is completed.
     */
    extern "C" mxf_tk_API uint32_t CALLING_CONVENTION NewP2Shot(locators* /* in free **/, 
                                                            const char* /* in free */, 
                                                            const char* /* in free */, 
                                                            p2_format,
                                                            I_timecode* = NULL /* in free */, 
                                                            I_umid* =NULL /* in free */);

    /* NewP2Shot
     * This function lets you fill a P2 "Contents" directory with MXF P2 files
     * similar to the ones the P2 Camcorder would produce.
     * The first five parameters are the streaming interface you should derive to provide an implementation 
     * to control your input streaming device. It should handle 1 DV stream and 2 or 4 AES streams. If only
     * 2 AES streams are being manipulated then the fourth and fifth I_essence_stream_task* should be set
     * to NULL.
     * The sixth parameter is the complete path toward the output "Contents" directory
     * The seventh parameter is the basename for this shot (that will be used
     * to set up the names of the MXF files) and should be EXACTLY 6 characters long (+ '\0' not counted)
     * API user can also set the start timecode and umid for this shot or leave this parameter to NULL
     * and the API will generate default ones.
     * The eigth parameter specifies the P2 format
     * The nineth parameter defines the start timecode of the sequence
     * The tenth parameter defines the umid of the generic material
     * Returns !=0 if the function operated sucessfully. The returned uint32_tshould then be used as parameter 
     * of WaitEndNewP2Shot, CancelNewP2Shot and ProgressNewP2Shot
     * WARNING! You should call WaitEndNewP2Shot after calling this function to make sure the process is completed.
     */
    extern "C" mxf_tk_API uint32_t CALLING_CONVENTION NewP2ShotFromStream(I_essence_stream_task* /* in free */,
								      I_essence_stream_task* /* in free */,
								      I_essence_stream_task* /* in free */,
								      I_essence_stream_task* /* in free */,
								      I_essence_stream_task* /* in free */,
								      const char* /* in free */, 
								      const char* /* in free */, 
                                                                      p2_format,
								      I_timecode* = NULL /* in free */, 
								      I_umid* =NULL /* in free */);

     /*
     * Wait for the end of the current P2 creation process.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION WaitEndNewP2Shot(uint32_t);

    /*
     * Cancel the current P2 creation process.
     * You must still call WaitEndNewP2Shot() to terminate correctly the thread.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION CancelNewP2Shot(uint32_t);

    /*
     * Return progress of the current P2 creation process.
     * Return value is comprised between 0.0 and 1.0 or is negative if the
     * progress could not be computed
     */
    extern "C" mxf_tk_API double CALLING_CONVENTION ProgressNewP2Shot(uint32_t);

    /* Build P2 XML
     * This function builds the P2 XML (stored in the Clip directory) from a P2 VIDEO file
     * The P2 file provided should be in the "Contents\Video" directory structure and all its
     * associated audio files should be in Contents\Audio as well. You do not need to call this function
     * if you already have the XML file. The first use of this function is to rebuild the XML clip file
     * after performing a partial restore on a set of P2 files.
     * Returns true if the function operated successfully.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION BuildP2XML(const char* /* in free */, p2_format /* in */);

    /* Build P2 XML
     * This function builds the P2 XML buffer (which should be flushed in the Clip directory) from a 
     * P2 assembled I_mxf_file (first parameter). 
     * First parameter is a P2 MXF file (already linked to its associated P2 files thanks to opatom_assembler)
     * Second parameter is the clip name of the P2 set of file
     * Third paramter is the buffer size, this value is set to the XML size
     * Last parameter is the buffer
     * Returns true if the function operated successfully.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION BuildP2XMLBuffer(I_mxf_file * /* in free */, 
                                                                   const char * /* in free */, 
                                                                   size_t * /* out free */, 
                                                                   unsigned char * /* out free */,
                                                                   p2_format /* in */);


    /* Return the P2 format from an XML file (in the Clip directory)
     * The parameter is the path toward the XML file.
    */
    extern "C" mxf_tk_API p2_format CALLING_CONVENTION GetP2Format(const char* /* in free */);
    

    class I_opatom_file;
		
    /* I_opatom_assembler
     * An interface to manipulate several opatom files.
     * When creating opatom files, it provides methods for parallelizing or serializing opatom files.
     * In that case the generic material of the opatom files is updated.
     * When reading opatom files, it provides a method for linking the files thanks to their umid 
     * and rebuild the entire generic material structure.
     */
    class I_opatom_assembler
    {
    public:

      // ****************************** WRITING *************************************

      /*
       * This function parallelizes two op_atom files so that their embedded essence
       * are meant to be played together.
       * This is an associative function -> if file1 is parallelized with file2
       * and file 1 is parralelized with file 3 then file2 and file3 are 
       * automatically parallelized.
       * Returns true if successful
       */
      virtual bool CALLING_CONVENTION parallelize(I_opatom_file* /* in free */, 
                                                  I_opatom_file* /* in free */)=0;

      /*
       * This function serializes two op_atom files so that their embedded essence
       * will be played consecutively on the same track.
       * Returns true if successful
       */
      virtual bool CALLING_CONVENTION serialize(I_opatom_file* /* in free */, 
                                                I_opatom_file* /* in free */)=0;

      // ****************************** READING **************************************

      /* 
       * This function synchronizes several opatom files that were created together.
       * In order to get the full fonctionality from the opatom structure, you
       * should call this function first before manipulating each file individually.
       * Returns true if successful.
       */
      virtual bool CALLING_CONVENTION synchronize(mxf_file_list* /* in free */)=0;
    };
	
    /*
     * Retrieves an I_opatom_assembler interface 
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewOpAtomAssemblerInterface(I_opatom_assembler** /* out free */);

    /*
     * Frees an I_opatom_assembler interface.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeOpAtomAssemblerInterface(I_opatom_assembler** /* out free */);


    /* I_opatom_file:
     * An interface to create files following the OpAtom pattern requirement
     * An OpAtom file contains a single track, therefore if several tracks are to be recorded
     * several OpAtom files will be written when calling flush()
     */
    class I_opatom_file : public I_mxf_file
    {
    private:
      virtual size_t CALLING_CONVENTION write(const uint8_t*, size_t)=0;
      virtual bool CALLING_CONVENTION end_of_stream()=0;
    public:
      /*
       * Sets the material to be wrapped in an mxf_file.
       * Output material will also be automatically created upon calling of this function.
       * Returns true if successful.
       */
      virtual bool CALLING_CONVENTION set_material(I_concrete_material* /* in */)	=0;

      /*
       * Returns the umid uniquely identifying the essence embedded in
       * this opatom. 
       * Returned pointer must be destroyed by the API user (FreeUmidInterface).
       * Returns NULL if an error occured.
       */
      virtual I_umid* /* free */ CALLING_CONVENTION get_concrete_material_umid() const=0;
    };

    /*
     * Retrieves an I_atom_op1a_file interface.
     * I_atom_file** pointer to the newly allocated object
     * const char* path to the mxf file to be written. 
     * Call to flush() will effectively cause the on-disk writing of
     * the file(s). If several files are written then their umid will be included
     * to the filename
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewOpAtomFileInterface(I_opatom_file** /* out free */, 
                                                                         const char* /* in free */);

    /*
     * Retrieves an I_atom_op1a_file interface.
     * I_atom_file** pointer to the newly allocated object
     * const char* path to the mxf file to be written. 
     * DCP is a constraint to OpAtom. You won't be 
     * able to change the parameters of the file.
     * Call to flush() will effectively cause the on-disk writing of
     * the file(s). If several files are written then their umid will be included
     * to the filename
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewDCPOpAtomFileInterface(I_opatom_file** /* out free */, 
                                                                            const char* /* in free */);

    /*
     * Retrieves an I_opatom_file interface whose flush will be performed in streaming mode.
     * I_opatom_file** pointer to the newly allocated object
     * const char* path to the mxf file to be written.
     * Call to flush() will effectively cause the on-disk writing of
     * the file(s).
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewOpAtomStreamInterface(I_opatom_file** /* out free */, 
                                                                           I_output_mxf_stream_task* /* in free */);

    /*
     * Retrieves an I_opatom_file interface whose flush will be performed in streaming mode.
     * I_opatom_file** pointer to the newly allocated object
     * DCP is a constraint to OpAtom. You won't be 
     * able to change the parameters of the file.
     * The streaming interface you provide SHALL implement the interfaces in order to seek
     * in the output MXF stream. 
     * const char* path to the mxf file to be written.
     * Call to flush() will effectively cause the on-disk writing of
     * the file(s).
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewDCPOpAtomStreamInterface(I_opatom_file** /* out free */, 
                                                                              I_output_mxf_stream_task* /* in free */);

    /*
     * Frees an I_opatom_file interface.
     * I_opatom_file** pointer to the object to be freed.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeOpAtomFileInterface(I_opatom_file** /* in out */);
  }
}

#endif
