//------------------------------------------------------------------------------
//  Author:    	Thierry FRANZETTI
//  Comment:	MXF_TK: A simple example to illustrate metadata update of an existing
//				file
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#if defined(BC6)	// Borland C Builder 6.0
#include <vcl.h>
#endif

#include <mxf_tk/I_mxf_error.hpp>
#include <mxf_tk/I_mxf_file.hpp>
#include <mxf_tk/I_metadata_material.hpp>

#include <iostream>

using namespace opencube::MXFTK_NAMESPACE;
using namespace std;

// MxfTk error handler
I_mxf_error_handler* err_h = NULL;

//Throws an exception when an error occured 
void throw_on_error(); 

//Called if an error occured.
void display_errors();

int main(int argc, char* argv[])
{
  if (argc < 2)
  {
    wcout << L"usage : in.mxf" << endl;
    exit(0);
  }

  I_mxf_file* pi_mxf_file = NULL;

  try
  {
    wcout << L"Retrieve MxfTk Error Handler" << endl;
    if (!GetMxfErrorHandlerInterface(&err_h))
      throw 1;

    wcout << L"MxfTk Initialization" << endl;
    InitMXF_TK();
    throw_on_error();

    wcout << L"Loading " << argv[1] << endl;
    NewMxfFileInterface(&pi_mxf_file, argv[1]);
    throw_on_error();

    // for demonstration purpose we remove all the existing metadata
    // tracks and replace them with a simple static one.
    // However MXFTk let's you add an remove not only the whole trakcs
    // but also add/remove metadata_material and metadata sets.
    wcout << L"Removing previous metadata tracks" << endl;
    // loop through output material
    generic_list* genl = pi_mxf_file->output_material();
    I_generic_material** p_gen_mat = NULL;
    while ((p_gen_mat = genl->next()) != NULL)
    {
      I_generic_material* gen_mat = *p_gen_mat;
      // loop through metadata tracks
      track_list* trl = gen_mat->metadata_tracks();
      if (trl)
      {
        I_track** p_tr = NULL;
        while ((p_tr = trl->next()) != NULL)
        {
          I_track* tr = *p_tr;
          // remove metadata track
          gen_mat->delete_metadata_track(tr);
        }
        gen_mat->free_tracks(trl);
      }
    }
    throw_on_error();

    // add a simple metadata track to the first material
    genl->begin();
    I_generic_material* first_mat = *genl->next();
    pi_mxf_file->free_generic_list(genl);
    rational* pi_rat = NULL;
    NewRationalInterface(&pi_rat, 25, 1);
    I_track* new_track = first_mat->new_metadata_track(timeline_metadata, 0, pi_rat);
    FreeRationalInterface(&pi_rat);
    throw_on_error();

    // Create a Production framework
    I_metadata* pr_fr = NULL;
    NewMetadataInterfaceByLabel(&pr_fr, "http://www.smpte-ra.org/schemes/434/200X/groups/S380M/2004:ProductionFramework");
    throw_on_error();
    // add a simple property
    wstring wstr = L"The file was updated!";
    ADD_PROPERTY(pr_fr, "http://www.smpte-ra.org/schemes/434/200X/properties/RP210:FrameworkTitle", MXF_UTF16_STRING, wstr.size()*sizeof(wchar_t), wstr.c_str());
    throw_on_error();

    // create the metadata material
    I_metadata_material* pi_meta_mat = NULL;
    NewMetadataMaterialInterface(&pi_meta_mat, // metata material
      m_timeline, // goes on a timeline track
      pr_fr, // production framework
      0, // origin
      -1, //duration will be automatically computed this way
      NULL, // no comment
      0); // size of comment = 0
    throw_on_error();

    // add the metadata material to the metadata track.
    new_track->add_metadata(pi_meta_mat, NULL);
    throw_on_error();
  }
  catch(int)
  {
    wcout << L"An error occured!" << endl;
    // Retrieving details of the error(s) and display it(them).
    display_errors();
    FreeMxfFileInterface(&pi_mxf_file);
    return -1;
  }

  // the file will be updated here
  try
  {
    if (pi_mxf_file)
    {
      UpdateAndFreeMxfFileInterface(&pi_mxf_file);
      throw_on_error();
    }
  }
  catch(int)
  {
    wcout << L"An error occured!" << endl;
    // Retrieving details of the error(s) and display it(them).
    display_errors();
  }

}

void throw_on_error()
{
  if (err_h->has_an_error())
    throw 1;
}

void display_errors()
{
  // Retrieve error handler
  if (!err_h)
    return;

  // Retrieve list of errors
  error_list* err_l = err_h->get_errors();
  if (err_l)
  {
    err_l->begin();
    I_mxf_error** ppi_mxf_error = NULL;
    while ((ppi_mxf_error = err_l->next()) != NULL) //go through the list
    {
      I_mxf_error* pi_error = *ppi_mxf_error;
      // and display each error 
      wcout << L"Error " << (int)pi_error->get_id() << L": " << pi_error->get_error_msg() << endl;
    }
    // free error list interface
    err_h->free_errors(err_l);
  }
}
