//------------------------------------------------------------------------------
//  Author:    	Thierry FRANZETTI
//  Comment:	MXF_TK: A simple example to store the header metadata of a file
//  in an xml file. Also illustrates opening of an mxf file for reading.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#if defined(BC6)	// Borland C Builder 6.0
#include <vcl.h>
#endif

#include <mxf_tk/I_mxf_error.hpp>
#include <mxf_tk/I_mxf_file.hpp>

#include <iostream>

using namespace opencube::MXFTK_NAMESPACE;
using namespace std;

// MxfTk error handler
I_mxf_error_handler* err_h = NULL;

//Throws an exception when an error occured 
void throw_on_error(); 

//Called if an error occured.
void display_errors();

int main(int argc, char* argv[])
{
  if (argc < 3)
  {
    wcout << L"usage : out.xml in.mxf" << endl;
    exit(0);
  }

  I_mxf_file* pi_mxf_file = NULL;

  try
  {
    wcout << L"Retrieve MxfTk Error Handler" << endl;
    if (!GetMxfErrorHandlerInterface(&err_h))
      throw 1;

    wcout << L"MxfTk Initialization" << endl;
    InitMXF_TK();
    throw_on_error();

    wcout << L"Loading " << argv[2] << endl;
    NewMxfFileInterface(&pi_mxf_file, argv[2], CLOSED_COMPLETE_METADATA);
    throw_on_error();

    wcout << L"Output header metadata in an XML file" << endl;
    pi_mxf_file->output_xml(argv[1]);
    throw_on_error();
  }
  catch(int)
  {
    wcout << L"An error occured!" << endl;
    // Retrieving details of the error(s) and display it(them).
    display_errors();
  }
                     
  if (pi_mxf_file)
    FreeMxfFileInterface(&pi_mxf_file);
}

void throw_on_error()
{
  if (err_h->has_an_error())
    throw 1;
}

void display_errors()
{
  // Retrieve error handler
  if (!err_h)
    return;

  // Retrieve list of errors
  error_list* err_l = err_h->get_errors();
  if (err_l)
  {
    err_l->begin();
    I_mxf_error** ppi_mxf_error = NULL;
    while ((ppi_mxf_error = err_l->next()) != NULL) //go through the list
    {
      I_mxf_error* pi_error = *ppi_mxf_error;
      // and display each error 
      wcout << L"Error " << (int)pi_error->get_id() << L": " << pi_error->get_error_msg() << endl;
    }
    // free error list interface
    err_h->free_errors(err_l);
  }
}
